/*
 * Copyright (c) 2004 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef	PF_EST_SN_h_defined
#define	PF_EST_SN_h_defined
// ---------------------------------------------------------------------
#include "MICARYHA_struct.h"
#include "MICARY_PF_Struct.h"

// 2007/07/13 added by S.Suzuki calling C from C++
#ifdef __cplusplus
extern "C" {
#endif

// ---------------------------------------------------------------------

/**
 \brief \~Japanese SN比推定(1:正常終了, 0:エラー)
 \param fsInputPSpec \~English Input power spectrum (N, M)
 \~Japanese 入力パワー
 \param fsEstNoisePSpec \~English Total Estimated Noise spectrum (N, M)
 \~Japanese 妨害音パワー
 \param psSavedSN \~English Saved Valiables for S/N Calc.
 \~Japanese SN比保存値
 \param sPrmSN \~English parameters for S/N Calc.
 \~Japanese SN比計算パラメータ
 \param pfsGH1 \~English Optimal amplitude gain (N, M)
 \~Japanese 最適増幅係数
 \param pfsSN \~English Estimated postteriori ~= (SNR - 1)  (N, M)
 \~Japanese SN比推定値
 \param pfsEstVal \~English Variable for optimum gain estimator  (N, M)
 \~Japanese 最適ゲイン推定値算出のための変数
*/
int PF_EstSN(
			MICARY_FloatSpecArray fsInputPSpec,
			MICARY_FloatSpecArray fsEstNoisePSpec,
			MICARY_SavedEstSN *psSavedSN,
			MICARY_PrmEstSN sPrmSN,
			MICARY_FloatSpecArray *pfsGH1,
			MICARY_FloatSpecArray *pfsSN,
			MICARY_FloatSpecArray *pfsEstVal);

#ifdef __cplusplus
}
#endif

#endif	// PF_EST_SN_h_defined
