/*
 * Copyright (c) 2004 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef	PF_EST_VOICE_P_h_defined
#define	PF_EST_VOICE_P_h_defined
// ---------------------------------------------------------------------
#include "MICARYHA_struct.h"
#include "MICARY_PF_Struct.h"

// 2007/07/13 added by S.Suzuki calling C from C++
#ifdef __cplusplus
extern "C" {
#endif
// ---------------------------------------------------------------------

/**
 \brief \~Japanese 音声存在確率の推定(1:正常終了, 0:エラー)
 \param fsPreSN \~English Priori SNR (N, M)
 \~Japanese 事前SN比
 \param fsEstVal \~English Variable for optimum gain estimator  (N, M)
 \~Japanese 最適ゲイン推定値算出のための変数
 \param sPrmZeta \~English Parameters for Estimate speech present probability
 \~Japanese 音声存在確率計算のパラメータ
 \param psSavedVoiceP \~English Speech present probability of previous frame (N ,M)
 \~Japanese 前フレームの音声存在確率計算の結果
 \param pfsVoiceP \~English Probability of speech present (p) (N, M)
 \~Japanese 音声存在確率
 \param pfsSmSNR \~English Smoothed priori SNR (zeta) (N, M)
 \~Japanese 平滑化された事前SN比
 \param pfaSmSNRF \~English zeta_frame frame (M, 1)
 \~Japanese 平滑化された事前SN比(Frame)
 \param pfaSmSNRP \~English zeta_frame peak (M, 1)
 \~Japanese 平滑化された事前SN比(Peak)
*/
int PF_EstVoiceP(
		MICARY_FloatSpecArray fsPreSN,
		MICARY_FloatSpecArray fsEstVal,
		MICARY_PrmVoiceP sPrmVoiceP,
		MICARY_SavedVoiceP *psSavedVoiceP,
		MICARY_FloatSpecArray *pfsVoiceP,
		MICARY_FloatSpecArray *pfsSmSNR,
		MICARY_FloatArray *pfaSmSNRF,
		MICARY_FloatArray *pfaSmSNRP);

#ifdef __cplusplus
}
#endif
// --------------------------------------------------------------------

#endif	// PF_EST_VOICE_P_h_defined
