/*
 * Copyright (c) 2004 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef	PF_EST_ZETA_h_defined
#define	PF_EST_ZETA_h_defined
// ---------------------------------------------------------------------
#include "MICARYHA_struct.h"
#include "MICARY_PF_Struct.h"

// 2007/07/13 added by S.Suzuki calling C from C++
#ifdef __cplusplus
extern "C" {
#endif
// ---------------------------------------------------------------------

/**
 \brief \~Japanese 事前SN比の平滑化(1:正常終了, 0:エラー)
 \param fsPreSN \~English Priori SNR (N, M)
 \~Japanese 事前SN比
 \param sPrmZeta \~English Parameters for smoothing calc.
 \~Japanese 平滑化計算のパラメータ
 \param pfsSavedSmSNR \~English Smoothed SNR of previous frame (N ,M)
 \~Japanese 前フレームの平滑化SN比
 \param pfsSmSNR \~English Smoothed priori SNR (zeta) (N, M)
 \~Japanese 平滑化された事前SN比
 \param pfsSmSNRLocal \~English Smoothed priori SNR (zetaL) (Local) (N, M)
 \~Japanese 平滑化された事前SN比(Local)
 \param pfsSmSNRGlobal \~English Smoothed priori SNR (zetaG) (Global) (N, M)
 \~Japanese 平滑化された事前SN比(Global)
 \param pfaSmSNRFrame \~English Smoothed priori SNR (zetaF) (Frame) (M, 1)
 \~Japanese 平滑化された事前SN比(Frame)
*/
int PF_EstZeta(
		MICARY_FloatSpecArray fsPreSN,
		MICARY_PrmZeta sPrmZeta,
		MICARY_FloatSpecArray fsSavedSmSNR,
		MICARY_FloatSpecArray *pfsSmSNR,
		MICARY_FloatSpecArray *pfsSmSNRL,
		MICARY_FloatSpecArray *pfsSmSNRG,
		MICARY_FloatArray *pfaSmSNRF);

#ifdef __cplusplus
}
#endif
// ---------------------------------------------------------------------

#endif	// PF_EST_ZETA_h_defined
